@@ -477,6 +477,8 @@ EX(CMD_keepmarks,	"keepmarks",	ex_wrongmodifier,
 			NEEDARG|EXTRA|NOTRLCOM),
 EX(CMD_keepjumps,	"keepjumps",	ex_wrongmodifier,
 			NEEDARG|EXTRA|NOTRLCOM),
+EX(CMD_keeppatterns,	"keeppatterns",	ex_wrongmodifier,
+			NEEDARG|EXTRA|NOTRLCOM),
 EX(CMD_keepalt,		"keepalt",	ex_wrongmodifier,
 			NEEDARG|EXTRA|NOTRLCOM),
 EX(CMD_list,		"list",		ex_print,
@@ -571,14 +573,14 @@ EX(CMD_lrewind,		"lrewind",	ex_cc,
 			RANGE|NOTADR|COUNT|TRLBAR|BANG),
 EX(CMD_ltag,		"ltag",	ex_tag,
 			NOTADR|TRLBAR|BANG|WORD1),
+EX(CMD_lunmap,		"lunmap",	ex_unmap,
+			EXTRA|TRLBAR|NOTRLCOM|USECTRLV|CMDWIN),
 EX(CMD_lua,		"lua",		ex_lua,
 			RANGE|EXTRA|NEEDARG|CMDWIN),
 EX(CMD_luado,		"luado",	ex_luado,
 			RANGE|DFLALL|EXTRA|NEEDARG|CMDWIN),
 EX(CMD_luafile,		"luafile",	ex_luafile,
 			RANGE|FILE1|NEEDARG|CMDWIN),
-EX(CMD_lunmap,		"lunmap",	ex_unmap,
-			EXTRA|TRLBAR|NOTRLCOM|USECTRLV|CMDWIN),
 EX(CMD_lvimgrep,	"lvimgrep",	ex_vimgrep,
 			RANGE|NOTADR|BANG|NEEDARG|EXTRA|NOTRLCOM|TRLBAR|XFILE),
 EX(CMD_lvimgrepadd,	"lvimgrepadd",	ex_vimgrep,
@@ -653,6 +655,8 @@ EX(CMD_noreabbrev,	"noreabbrev",	ex_abbreviate,
 			EXTRA|TRLBAR|NOTRLCOM|USECTRLV|CMDWIN),
 EX(CMD_noremenu,	"noremenu",	ex_menu,
 			RANGE|NOTADR|ZEROR|BANG|EXTRA|TRLBAR|NOTRLCOM|USECTRLV|CMDWIN),
+EX(CMD_noswapfile,	"noswapfile",	ex_wrongmodifier,
+			NEEDARG|EXTRA|NOTRLCOM),
 EX(CMD_normal,		"normal",	ex_normal,
 			RANGE|BANG|EXTRA|NEEDARG|NOTRLCOM|USECTRLV|SBOXOK|CMDWIN),
 EX(CMD_number,		"number",	ex_print,
@@ -662,7 +666,7 @@ EX(CMD_nunmap,		"nunmap",	ex_unmap,
 EX(CMD_nunmenu,		"nunmenu",	ex_menu,
 			EXTRA|TRLBAR|NOTRLCOM|USECTRLV|CMDWIN),
 EX(CMD_open,		"open",		ex_open,
-			RANGE|EXTRA),
+			RANGE|BANG|EXTRA),
 EX(CMD_oldfiles,	"oldfiles",	ex_oldfiles,
 			BANG|TRLBAR|SBOXOK|CMDWIN),
 EX(CMD_omap,		"omap",		ex_map,
@@ -739,10 +743,14 @@ EX(CMD_pwd,		"pwd",		ex_pwd,
 			TRLBAR|CMDWIN),
 EX(CMD_python,		"python",	ex_python,
 			RANGE|EXTRA|NEEDARG|CMDWIN),
+EX(CMD_pydo,		"pydo",		ex_pydo,
+			RANGE|DFLALL|EXTRA|NEEDARG|CMDWIN),
 EX(CMD_pyfile,		"pyfile",	ex_pyfile,
 			RANGE|FILE1|NEEDARG|CMDWIN),
 EX(CMD_py3,		"py3",		ex_py3,
 			RANGE|EXTRA|NEEDARG|CMDWIN),
+EX(CMD_py3do,		"py3do",	ex_py3do,
+			RANGE|DFLALL|EXTRA|NEEDARG|CMDWIN),
 EX(CMD_python3,		"python3",	ex_py3,
 			RANGE|EXTRA|NEEDARG|CMDWIN),
 EX(CMD_py3file,		"py3file",	ex_py3file,
@@ -832,9 +840,9 @@ EX(CMD_set,		"set",		ex_set,
 EX(CMD_setfiletype,	"setfiletype",	ex_setfiletype,
 			TRLBAR|EXTRA|NEEDARG|CMDWIN),
 EX(CMD_setglobal,	"setglobal",	ex_set,
-			TRLBAR|EXTRA|CMDWIN),
+			TRLBAR|EXTRA|CMDWIN|SBOXOK),
 EX(CMD_setlocal,	"setlocal",	ex_set,
-			TRLBAR|EXTRA|CMDWIN),
+			TRLBAR|EXTRA|CMDWIN|SBOXOK),
 EX(CMD_sfind,		"sfind",	ex_splitview,
 			BANG|FILE1|RANGE|NOTADR|EDITCMD|ARGOPT|TRLBAR),
 EX(CMD_sfirst,		"sfirst",	ex_rewind,
@@ -921,6 +929,8 @@ EX(CMD_swapname,	"swapname",	ex_swapname,
 			TRLBAR|CMDWIN),
 EX(CMD_syntax,		"syntax",	ex_syntax,
 			EXTRA|NOTRLCOM|CMDWIN),
+EX(CMD_syntime,		"syntime",	ex_syntime,
+			NEEDARG|WORD1|TRLBAR|CMDWIN),
 EX(CMD_syncbind,	"syncbind",	ex_syncbind,
 			TRLBAR),
 EX(CMD_t,		"t",		ex_copymove,
@@ -944,7 +954,7 @@ EX(CMD_tabfind,		"tabfind",	ex_splitview,
 EX(CMD_tabfirst,	"tabfirst",	ex_tabnext,
 			TRLBAR),
 EX(CMD_tabmove,		"tabmove",	ex_tabmove,
-			RANGE|NOTADR|ZEROR|COUNT|TRLBAR|ZEROR),
+			RANGE|NOTADR|ZEROR|EXTRA|NOSPC|TRLBAR),
 EX(CMD_tablast,		"tablast",	ex_tabnext,
 			TRLBAR),
 EX(CMD_tabnext,		"tabnext",	ex_tabnext,
